==> cursors_extended-fabric-4.0.5%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/mixin/CursorTypeMixin.java <==
package io.github.fishstiz.cursors_extended.mixin;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import io.github.fishstiz.cursors_extended.CursorsExtended;
import io.github.fishstiz.cursors_extended.cursor.CursorDisplay;
import io.github.fishstiz.cursors_extended.cursor.CursorRegistry;
import io.github.fishstiz.cursors_extended.cursor.CursorTypesExt;
import io.github.fishstiz.cursors_extended.cursor.TexturedCursorType;
import io.github.fishstiz.cursors_extended.resource.texture.CursorTexture;
import net.minecraft.class_1041;
import net.minecraft.class_11875;
import net.minecraft.class_11876;
import org.jetbrains.annotations.Nullable;
import org.lwjgl.glfw.GLFW;
import org.lwjgl.system.MemoryUtil;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

@Mixin(class_11875.class)
public abstract class CursorTypeMixin implements TexturedCursorType {
    @Shadow
    public abstract void select(class_1041 window);

    @Unique
    private CursorTexture cursors_extended$texture;

    static {
        CursorsExtended.LOGGER.debug("[cursors_extended] Initializing CursorTypes: {}", class_11876.class);
    }

    @Override
    public @Nullable CursorTexture cursors_extended$getTexture() {
        return cursors_extended$texture;
    }

    @Override
    public void cursors_extended$setTexture(CursorTexture texture) {
        CursorTexture previousTexture = cursors_extended$texture;
        this.cursors_extended$texture = texture;

        CursorDisplay cursorDisplay = CursorsExtended.getInstance().getDisplay();
        CursorRegistry cursorRegistry = CursorsExtended.getInstance().getRegistry();
        if (cursorDisplay.getDisplayedCursor() == cursorRegistry.get(((class_11875) (Object) this))) {
            select(cursorDisplay.getWindow());
        }

        if (previousTexture != null && previousTexture != texture) {
            previousTexture.close();
        }
    }

    @WrapOperation(method = "select", at = @At(
            value = "FIELD",
            target = "Lcom/mojang/blaze3d/platform/cursor/CursorType;handle:J"
    ))
    private long onSetCursor(class_11875 instance, Operation<Long> original) {
        if (cursors_extended$texture != null && cursors_extended$texture.handle() != MemoryUtil.NULL) {
            return cursors_extended$texture.handle();
        }

        return original.call(instance);
    }

    @Inject(method = "createStandardCursor", at = @At("RETURN"))
    private static void registerStandardCursors(int shape, String name, class_11875 fallback, CallbackInfoReturnable<class_11875> cir) {
        if (cir.getReturnValue() != null) {
            class_11875 standardCursorType = cursors_extended$mapStandardCursor(shape);
            if (standardCursorType != null) {
                CursorsExtended.LOGGER.info("[cursors_extended] Registering an alias for {}: {}", standardCursorType, name);
                CursorsExtended.getInstance().getRegistry().registerAlias(standardCursorType, cir.getReturnValue());
            }
        }
    }

    @Unique
    private static @Nullable class_11875 cursors_extended$mapStandardCursor(int shape) {
        return switch (shape) {
            case GLFW.GLFW_ARROW_CURSOR -> class_11875.field_62449;
            case GLFW.GLFW_POINTING_HAND_CURSOR -> class_11876.field_62455;
            case GLFW.GLFW_IBEAM_CURSOR -> class_11876.field_62453;
            case GLFW.GLFW_CROSSHAIR_CURSOR -> class_11876.field_62454;
            case GLFW.GLFW_RESIZE_EW_CURSOR -> class_11876.field_62457;
            case GLFW.GLFW_RESIZE_NS_CURSOR -> class_11876.field_62456;
            case GLFW.GLFW_RESIZE_NWSE_CURSOR -> CursorTypesExt.RESIZE_NWSE;
            case GLFW.GLFW_RESIZE_NESW_CURSOR -> CursorTypesExt.RESIZE_NESW;
            case GLFW.GLFW_RESIZE_ALL_CURSOR -> class_11876.field_62458;
            case GLFW.GLFW_NOT_ALLOWED_CURSOR -> class_11876.field_62459;
            default -> null;
        };
    }
}

==> cursors_extended-fabric-4.1.2%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/mixin/CursorTypeMixin.java <==
package io.github.fishstiz.cursors_extended.mixin;

import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import com.llamalad7.mixinextras.injector.wrapoperation.WrapOperation;
import io.github.fishstiz.cursors_extended.CursorsExtended;
import io.github.fishstiz.cursors_extended.cursor.Cursor;
import io.github.fishstiz.cursors_extended.cursor.CursorDisplay;
import io.github.fishstiz.cursors_extended.cursor.TexturedCursorType;
import io.github.fishstiz.cursors_extended.resource.texture.CursorTexture;
import io.github.fishstiz.cursors_extended.util.CursorTypeUtil;
import net.minecraft.class_1041;
import net.minecraft.class_11875;
import net.minecraft.class_11876;
import org.jetbrains.annotations.Nullable;
import org.lwjgl.system.MemoryUtil;
import org.spongepowered.asm.mixin.*;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

@Mixin(class_11875.class)
public abstract class CursorTypeMixin implements TexturedCursorType {
    @Shadow
    public abstract void select(class_1041 window);

    @Unique
    private CursorTexture cursors_extended$texture;

    static {
        CursorsExtended.LOGGER.debug("[cursors_extended] Loading CursorTypes: {}", class_11876.class);
    }

    @Override
    public @Nullable CursorTexture cursors_extended$getTexture() {
        return cursors_extended$texture;
    }

    @Unique
    private Cursor cursors_extended$getKey() {
        return CursorsExtended.getInstance().getRegistry().get((class_11875) (Object) this);
    }

    @Override
    public void cursors_extended$setTexture(CursorTexture texture) {
        CursorTexture previousTexture = cursors_extended$texture;
        this.cursors_extended$texture = texture;

        CursorDisplay cursorDisplay = CursorsExtended.getInstance().getDisplay();
        if (cursorDisplay.getDisplayedCursor() == cursors_extended$getKey()) {
            select(cursorDisplay.getWindow());
        }

        if (previousTexture != null && previousTexture != texture) {
            previousTexture.close();
        }
    }

    @WrapOperation(method = "select", at = @At(
            value = "FIELD",
            target = "Lcom/mojang/blaze3d/platform/cursor/CursorType;handle:J"
    ))
    private long onSetCursor(class_11875 instance, Operation<Long> original) {
        if (cursors_extended$texture != null &&
            cursors_extended$texture.handle() != MemoryUtil.NULL &&
            CursorsExtended.CONFIG.getOrCreateSettings(cursors_extended$getKey()).enabled()) {
            return cursors_extended$texture.handle();
        }
        return original.call(instance);
    }

    @Inject(method = "createStandardCursor", at = @At("RETURN"))
    private static void registerStandardCursors(int shape, String name, class_11875 fallback, CallbackInfoReturnable<class_11875> cir) {
        if (cir.getReturnValue() != null) {
            class_11875 standardCursorType = CursorTypeUtil.mapStandardCursor(shape);
            if (standardCursorType != null) {
                CursorsExtended.LOGGER.info("[cursors_extended] Registering an alias for {}: {}", standardCursorType, name);
                CursorsExtended.getInstance().getRegistry().registerAlias(standardCursorType, cir.getReturnValue());
            }
        }
    }
}
